;;; rx_tx.asm
;;;
;;; Bus Device for the Handy Cricket
;;; 
;;; pin assignments:
;;; Port B 0 is bus I/O, Port B7 is toggle pin
;;; 
;;; bus values:
;;; 17 (decimal) is bus address
;;; 100 or 101 are return values based on toggle pin's state.
;;; 
;;; NOTE!
;;; this bus code requires 4 MHz crystal with 1 usec per instruction clock

;	device pic16f84
;	config  OSC=xt
	radix dec
	
;;; register destination constants
W       equ     0
F       equ     1
	
;;; register constants
INDF	equ 0	
STATUS	equ 3			; status register
C	equ 0			; status C bit
Z	equ 2			; status Z bit
RP0	equ 5			; status register page 0
FSR	equ 4	

PORTA	equ 5			; port A register
PORTB	equ 6			; port B register

;;; bus port and pin
BUSPORT	equ PORTB
BUSTRIS	equ PORTB + 0x80	; for setting bus as output

PIN0	equ 0
PIN1	equ 1
PIN2	equ 2
PIN3 	equ 3
PIN4	equ 4
PIN5	equ 5
PIN6 	equ 6
PIN7	equ 7

;;; memory variables
buscntr equ 0x20		; bit counter 
busdata	equ 0x21		; bus data register
counter equ 0x22		; temp register

	org 0
	
start:
	bsf STATUS,RP0
;
; set-up pins for outputs
;
	bcf PORTA,PIN0		; make PORTA pin0 an output ( LED #1)
	bcf PORTA,PIN1		; make PORTA pin1 an output ( LED #2) 
 	bcf PORTB,PIN0		; make PORTB pin0 an output ( cricket bus comm )
	bcf PORTB,PIN4		; make PORTB pin4 an output ( CTR1 pin )
	bcf PORTB,PIN5		; make PORTB pin5 an output ( CTR0 pin )
	bcf PORTB,PIN7		; make PORTB pin7 an output ( Tx Line )
;
; set-up pins for inputs
;
	
	bsf PORTB,PIN6		; make PORTB pin6 an input ( Rx Line )
	bcf STATUS,RP0

	bcf PORTA,PIN0		; turn off LED #1
	bcf PORTA,PIN1		; turn off LED #2

;;; the main loop
;;; block until receives word on bus
;;; check if it's cmd or data, if data, ignore.
;;; if cmd, check if it's our magic number, if not, ignore
;;; if it's for us, toggle output pin then generate reply.
loop:	
	
	call bus_tyi		; get word from bus
	btfss STATUS,C		; if C set, it's a cmd word
	goto loop

	movf busdata,W		; move the data to busdata
	xorlw 10		; 10 is our magic number
	btfss STATUS,Z
	goto num2		; if Z clear, wasn't our number
	goto found 

num2:	movf busdata,W
	xorlw 11
	btfss STATUS,Z
	goto num3
	goto found
	
num3:	movf busdata,W
	xorlw 12
	btfss STATUS,Z
	goto num4
	goto found

num4:	movf busdata,W
	xorlw 13
	btfss STATUS,Z
	goto num5
	goto found

num5:	movf busdata,W
	xorlw 14
	btfss STATUS,Z
	goto num6
	goto found

num6:	movf busdata,W
	xorlw 15
	btfss STATUS,Z
	goto num7
	goto found

num7:	movf busdata,W
	xorlw 16
	btfss STATUS,Z
	goto num8
	goto found

num8:	movf busdata,W
	xorlw 17
	btfss STATUS,Z
	goto num9
	goto found

num9:	movf busdata,W
	xorlw 18
	btfss STATUS,Z
	goto num10
	goto found

num10:	movf busdata,W
	xorlw 19
	btfss STATUS,Z
	goto loop
	goto found

	;; toggle PORTA pin0 so we'll know cmd word was received!
found:	call toggle

	movlw busdata		; load the command to send to the cricket
	call bus_tyo		; now return it.
	
	goto loop
	
;;; inverts state of PORTA, pin0
toggle:
	btfsc PORTA,PIN0
	goto togclr
	bsf PORTA,PIN0
	return
togclr:	bcf PORTA,PIN0
	return

;;; inverts state of PORTA,pin1
toggle2:
	btfsc PORTA,PIN1
	goto togclr2
	bsf PORTA,PIN1
	return
togclr2:bcf PORTA,PIN1
	return

	
;;; returns byte in busdata
;;; the inverse of the stop bit in the carry
;;; commands have a 0 stop bit -> carry set
;;; data has a 1 stop bit -> carry clear

bus_tyi:

	bsf PORTB,PIN4		; Set transceiver to receive
	bsf PORTB,PIN5		;  transmission mode
	bcf PORTB,PIN7		; Set TX low for receiver operation
btyi10:
	btfsc BUSPORT,PIN6
	goto btyi10		; loop until bus line goes low

;	bcf INTCON,GIE		; disable interrupts (if necessary)

btyi20:	

	btfss BUSPORT,PIN6
	goto btyi20		; wait for end of prestart

	movlw 8
	movwf buscntr
	call an_rts		; use 4 cycles, into middle of start bit

btyi30:	
	nop
	nop
	nop
	rrf busdata,F		; pre-rotate destination byte
	bcf busdata,7		; clear high bit (it'll rotate down)
	btfsc BUSPORT,PIN6
	bsf busdata,7		; if PIN6 was set, set busdata bit
	decfsz buscntr,F
	goto btyi30

	call an_rts
	nop
	bsf STATUS,C		; pre-set carry (cmd word)
	btfsc BUSPORT,PIN6
	bcf STATUS,C		; if no stop bit, clear carry

;	bsf INTCON,GIE		; re-enable interrupts (if necessary)

an_rts:	return

;;; send a byte down the bus.
;;; this is the slave device code, so send '1' as stop bit,
;;;   indicating a data word.
;;; put the byte to send in busdata before calling.
bus_tyo:
;	bcf INTCON,GIE		; disable interrupts (if necessary)

	bsf PORTB,PIN4		; Set transceiver to ASK
	bcf PORTB,PIN5		;  transmission mode

	movlw BUSTRIS
	movwf FSR		; will be controlling bus to output
	bcf BUSPORT,PIN0	; put 0 into pin register
	bcf INDF,PIN0		; write it out & begin pre-start pulse
	movlw 33
	movwf counter
btyo20:	
	decfsz counter,F
	goto btyo20		; wait 100 usec
	bsf BUSPORT,PIN0	; end pre-start, begin start bit

	bsf STATUS,C		; will be stop bit, indicating data word
	movlw 9			; 8 data bits plus C bit last
	movwf counter
	call an_rts		; use up 4 cycles
	nop			; plus 1 more making 10 for start bit
	
btyo50:	
	rrf busdata,F		; bit -> carry
	btfss STATUS,C		; if C is clear, 
	bcf BUSPORT,PIN0	;   clear bus pin
	btfsc STATUS,C		; if C was set,
	bsf BUSPORT,PIN0	;   set bus pin
	nop
	nop			; tune the loop to take 10 us
	decfsz counter,F
	goto btyo50
	nop
	nop
	bsf INDF,PIN0		; bus to input again, float high
;	bsf INTCON,GIE		; re-enable interrupts (if necessary)
	return

	end


