 title  "lego_rs - Simple PICMicro RS-232 Interface"
;
;  This Application executes a "Bit-Banging" RS-232 Interface that
;	emmulates the LEGO Tower. The data is sent from a Serial Port
;	then transmitted via RF to another PIC. Once the receiving PIC
;	receives the transmission, a valid transmission is sent back to 
;	verify that the TX/RX link is working.
;
;
;  Hardware Notes:
;   PIC16F84 Running at 4 MHz
;   _MCLR is Pulled Up
;   PORTB.3 is the Transmit Output
;   PORTB.0 is the RS-232 Input
;
;
  LIST R=DEC
  INCLUDE "p16f84.inc"

;  Register Usage
 CBLOCK 0x020                   ;  Start Registers at End of the Values
Byte, Count			;  Variables for RS-232
Dlay				;  Dlay Count
 ENDC

#define  RX PORTB, 0		; Data Received from RS232 CPU Port
#define  LED2 PORTB, 1		; Test LED2
#define  LED1 PORTB, 2		; Test LED1
#define  TX PORTB, 3		; Data Sent to RS232 CPU Port
#define  CTR1 PORTB, 4		; DR3000 ctr1 pin
#define  CTR0 PORTB, 5		; DR3000 ctr0 pin
#define  RF_RX PORTB, 6		; DR3000 RF receive pin
#define  RF_TX PORTB, 7		; DR3000 RF transmit pin 
				;  (must be help low in receive mode!)

 PAGE
 __CONFIG _CP_OFF & _XT_OSC & _PWRTE_ON  & _WDT_OFF

;  Mainline of lego_rs
  org    0
 
  bsf    STATUS, RP0

; Set-up pins for correct directions
; Output Pins

  bcf    TX			;  Enable TX for Output
  bcf 	 LED1
  bcf    LED2
  bcf	 CTR1
  bcf	 CTR0
  bcf    RF_TX

; Input Pins
	
  bsf	 RX
  bsf    RF_RX

  bcf    STATUS, RP0

  bsf    LED1			; Turn On LED1 to test start of program
  bsf  	 LED2			; Turn On LED2 to test start of program 
  bcf	 RF_TX			; Set RF_TX low for receiving mode
  bsf	 CTR1			; Set DR3000 to
  bsf	 CTR0			;  receive mode

Loop

  btfss  RX			; Look for RX Start Bit
   goto RF_test
   call HalfBitDlay
   goto RX_test

RF_test
  btfss	 RF_RX			; Look for RF Start bit
   goto Loop
   call HalfBitDlay
   goto RF_test2

RX_test
  btfss  RX			; Test for a False Start Bit
   goto Loop
   goto RXLoop

RF_test2
  btfss	 RF_RX			; Test for a False Start Bit
   goto  Loop
   goto  RF_RXLoop  

; Receive Byte From the CPU Port 

RXLoop				;  Loop Here to Read in the Byte

  movlw  8			; Set Loop for 8 bits
  movwf  Count

RX_inner

  call   BitDlay		;  Wait a Full Byte

  bcf    STATUS, C		;  Set Carry Accordingly
  btfss  RX			;  Bit High or Low?
   bsf   STATUS, C

  rrf    Byte, f		;  Shift in the Byte

  goto   $ + 1			;  Make 11 Cycles in Loop
  goto   $ + 1

  decfsz Count, f
   goto  RX_inner

  call   BitDlay		;  Make Sure there is a "Stop" Bit

  btfsc  RX
   goto  Loop			;  Not High, Then No Byte

  movf   Byte, w		;  Change Byte to Upper Case
  addlw  255 - 'z'              ;  Get the High limit
  addlw  'z' - 'a' + 1          ;  Add Lower Limit to Set Carry
  btfss  STATUS, C              ;  If Carry Set, then Lower Case
  addlw  h'20'                  ;   Carry NOT Set, Restore Character
  addlw  'A'                    ;  Add 'A' to restore the Character
  movwf  Byte

  movlw  10			;  Send Upper Case Back with Start and Stop
  movwf  Count
  bcf    STATUS, C

   call toggle			; Toggle TX (LED) for test

   goto RF_TXLoop

; Receive Byte From the RF Pin 

RF_RXLoop			;  Loop Here to Read in the Byte

  movlw  8			; set Loop for 8 bits
  movwf  Count

RF_RX_inner

  call   BitDlay		;  Wait a Full Byte

  bcf    STATUS, C		;  Set Carry Accordingly
  btfss  RF_RX			;  Bit High or Low?
   bsf   STATUS, C

  rrf    Byte, f		;  Shift in the Byte

  goto   $ + 1			;  Make 11 Cycles in Loop
  goto   $ + 1

  decfsz Count, f
   goto  RF_RX_inner

  call   BitDlay		;  Make Sure there is a "Stop" Bit

  btfsc  RF_RX
   goto  Loop			;  Not High, Then No Byte

  movf   Byte, w		;  Change Byte to Upper Case
  addlw  255 - 'z'              ;  Get the High limit
  addlw  'z' - 'a' + 1          ;  Add Lower Limit to Set Carry
  btfss  STATUS, C              ;  If Carry Set, then Lower Case
  addlw  h'20'                  ;   Carry NOT Set, Restore Character
  addlw  'A'                    ;  Add 'A' to restore the Character
  movwf  Byte

  movlw  10			;  Send Upper Case Back with Start and Stop
  movwf  Count
  bcf    STATUS, C

  call 	 toggle2

   goto TXLoop

TXLoop

  bsf	 CTR1			; Set DR3000 to 
  bcf    CTR0			;  ASK Transmit mode

  btfsc  STATUS, C		;  Send the Bit in "Carry"
   goto  $ + 4
  nop				;  Send a "Low"
  bcf    RF_TX
  goto   $ + 3
  bsf    RF_TX			;  Send a "High"
  goto   $ + 1			;  6 Cycles in Loop

  call   BitDlay		;  Wait a Bit

  bsf    STATUS, C		;  Shift Out the Next Bit into Carry
  rrf    Byte, f

  decfsz Count, f		;  11 Intrinsic Delays in TXLoop
   goto  TXLoop

  bcf	 RF_TX			; Reset DR3000 back to receive mode
  bsf	 CTR1			; Set DR3000 to
  bsf	 CTR0			;  receive mode

  goto   Loop

RF_TXLoop

  bsf	 CTR1			; Set DR3000 to 
  bcf    CTR0			;  ASK Transmit mode

  btfsc  STATUS, C		;  Send the Bit in "Carry"
   goto  $ + 4
  nop				;  Send a "Low"
  bcf    RF_TX
  goto   $ + 3
  bsf    RF_TX			;  Send a "High"
  goto   $ + 1			;  6 Cycles in Loop

  call   BitDlay		;  Wait a Bit

  bsf    STATUS, C		;  Shift Out the Next Bit into Carry
  rrf    Byte, f

  decfsz Count, f		;  11 Intrinsic Delays in TXLoop
   goto  RF_TXLoop

  bcf	 RF_TX			; Reset DR3000 back to receive mode
  bsf	 CTR1			; Set DR3000 to
  bsf	 CTR0			;  receive mode

  goto   Loop
;******************************************************************************
BitDlay				;  Delay 833 - 15 Cycles (including Call/Return)

  movlw  204
  addlw  0x0FF			;  Take 1 Away from the Loop
  btfss  STATUS, Z
   goto  $ - 2

  goto   $ + 1

  return
;******************************************************************************

;******************************************************************************
HalfBitDlay			;  Delay (833 - 15) / 2 Cycles 

  movlw  100
  addlw  0x0FF			;  Take 1 Away from the Loop
  btfss  STATUS, Z
   goto  $ - 2

  return
;******************************************************************************
toggle

  btfsc LED1
  goto togclr
  bsf LED1
  return

togclr

  bcf LED1
  return
;******************************************************************************
toggle2

  btfsc LED2
  goto togclr2
  bsf LED2
  return

togclr2

  bcf LED2
  return
;******************************************************************************

 end
